// ---------------------------------------------------------------------------
// - Direct.hpp                                                              -
// - afnix:mth module - direct linear solver abstract class definition       -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2014 amaury darsch                                   -
// ---------------------------------------------------------------------------

#ifndef  AFNIX_DIRECT_HPP
#define  AFNIX_DIRECT_HPP

#ifndef  AFNIX_RPI_HPP
#include "Rpi.hpp"
#endif

#ifndef  AFNIX_SOLVER_HPP
#include "Solver.hpp"
#endif

namespace afnix {

  /// The Direct class is an abstract class that is part of the linear solver
  /// family. The class encapsulates the direct solver family.
  /// @author amaury darsch

  class Direct : public Solver {
  protected:
    /// the permutation
    Rpi* p_rpi;

  public:
    /// create a direct solver
    Direct (void);

    /// destroy this solver
    ~Direct (void);

    /// set the solver lhs
    /// @param lhs the left handside
    void setlhs (Rmi* lhs);

    /// set the solver permutation
    /// @param rpi the permutation object
    virtual void setrpi (Rpi* rpi);

    /// @return the solver permutation
    virtual Rpi* getrpi (void) const;

    /// factorize the lhs
    virtual void factorize (void);

    /// factorize the lhs by range
    /// @param row the starting row
    /// @param rng thr row range
    virtual void factorize (const t_long row, const t_long rng) =0;

    /// update a factorization by lhs range
    /// @param row the starting row
    /// @param rng the row range
    virtual void update (const t_long row, const t_long nru) =0;

    /// update a factorization by vectors
    /// @param u the vector argument
    /// @param v the vector argument
    virtual void update (const Rvi& u, const Rvi& v) =0;

  public:
    /// @return true if the given quark is defined
    bool isquark (const long quark, const bool hflg) const;
    
    /// apply this object with a set of arguments and a quark
    /// @param robj  the current runnable
    /// @param nset  the current nameset    
    /// @param quark the quark to apply these arguments
    /// @param argv  the arguments  to apply
    Object* apply (Runnable* robj, Nameset* nset, const long quark,
                   Vector* argv);
  };
}

#endif
