// ---------------------------------------------------------------------------
// - TlsHandshake.hpp                                                        -
// - afnix:tls service - tls handshake class definition                      -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2014 amaury darsch                                   -
// ---------------------------------------------------------------------------

#ifndef  AFNIX_TLSHANDSHAKE_HPP
#define  AFNIX_TLSHANDSHAKE_HPP

#ifndef  AFNIX_TLSMESSAGE_HPP
#include "TlsMessage.hpp"
#endif

namespace afnix {

  /// The TlsHandshake class is the handshake class for the tls protocol.
  /// A handshake class is designed to hold one or several message. Since
  /// multiple messages might be present, the class provides a mechanism
  /// to retreive them, one after another.
  /// @author amaury darsch

  class TlsHandshake : public TlsMessage {
  protected:
    /// the message index
    long d_midx;

  public:
    /// create an empty handshake
    TlsHandshake (void);

    /// create a handshake by record
    /// @param rcd the tls record
    TlsHandshake (TlsRecord* rcs);

    /// copy construct this handshake
    /// @param that the object to copy
    TlsHandshake (const TlsHandshake& that);

    /// assign a record to this one
    /// @param that the record to assign
    TlsHandshake& operator = (const TlsHandshake& that);

    /// @return the class name
    String repr (void) const;

    /// @return a clone of this object
    Object* clone (void) const;

    /// reset this handshake
    void reset (void);

  public:
    /// create a new object in a generic way
    /// @param argv the argument vector
    static Object* mknew (Vector* argv);
  };
}

#endif
