// ---------------------------------------------------------------------------
// - TlsMessage.hpp                                                          -
// - afnix:tls service - tls message record class definition                 -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2014 amaury darsch                                   -
// ---------------------------------------------------------------------------

#ifndef  AFNIX_TLSMESSAGE_HPP
#define  AFNIX_TLSMESSAGE_HPP

#ifndef  AFNIX_TLSRECORD_HPP
#include "TlsRecord.hpp"
#endif

namespace afnix {

  /// The TlsMessae class is the base class for the tls messages which
  /// encapsulates the behavior of the record object. BY storing a record
  /// the message data are kept localised to the original record.
  /// @author amaury darsch

  class TlsMessage : public TlsHeader {
  protected:
    /// the message record
    TlsRecord* p_rcd;
    
  public:
    /// create an empty message
    TlsMessage (void);

    /// create a message by record
    /// @para, rcd the message record
    TlsMessage (TlsRecord* rcd);

    /// copy construct this message
    /// @param that the object to copy
    TlsMessage (const TlsMessage& that);

    /// destroy this message
    ~TlsMessage (void);

    /// assign a message to this one
    /// @param that the header to assign
    TlsMessage& operator = (const TlsMessage& that);

    /// @return the class name
    String repr (void) const;

    /// @return a clone of this object
    Object* clone (void) const;

    /// reset this message
    void reset (void);

    /// @return the message type
    t_byte gettype (void) const;

    /// @return the message major version
    t_byte getmajor (void) const;

    /// @return the message minor version
    t_byte getminor (void) const;

    /// @return the message length
    long length (void) const;

  public:
    /// create a new object in a generic way
    /// @param argv the argument vector
    static Object* mknew (Vector* argv);
  };
}

#endif
