// ---------------------------------------------------------------------------
// - TlsSocket.hpp                                                           -
// - afnix:tls service - tls socket class definition                         -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2014 amaury darsch                                   -
// ---------------------------------------------------------------------------

#ifndef  AFNIX_TLSSOCKET_HPP
#define  AFNIX_TLSSOCKET_HPP

#ifndef  AFNIX_TCPSOCKET_HPP
#include "TcpSocket.hpp"
#endif

#ifndef  AFNIX_TLSRECORD_HPP
#include "TlsRecord.hpp"
#endif

namespace afnix {

  /// The TlsSocket class is the common socket class for the tls protocol.
  /// the class is designed to encapsulate the necessary information to
  /// encode and decode a stream. The tls socket class is not designed to
  /// negotatiate the security, and thus acts as a low level transport.
  /// @author amaury darsch

  class TlsSocket : public TcpSocket {
  public:
    /// create a tls socket by tcp socket
    /// @param s the tcp socket
    TlsSocket (TcpSocket* s);

    /// @return the class name
    String repr (void) const;

    /// get the next available record
    TlsRecord* getrecord (void);

  private:
    // make the copy constructor private
    TlsSocket (const TlsSocket&);
    // make the assignment operator private
    TlsSocket& operator = (const TlsSocket&);

  public:
    /// create a new object in a generic way
    /// @param argv the argument vector
    static Object* mknew (Vector* argv);
    
    /// @return true if the given quark is defined
    bool isquark (const long quark, const bool hflg) const;
    
    /// apply this object with a set of arguments and a quark
    /// @param robj  the current runnable
    /// @param nset  the current nameset    
    /// @param quark the quark to apply these arguments
    /// @param argv  the arguments to apply
    Object* apply (Runnable* robj, Nameset* nset, const long quark,
		   Vector* argv);
  };
}

#endif
